"""
default exception
"""
je_editor_error = "JEditor error"
"""
exec exception
"""
je_editor_exec_error = "JEditor exec error"
file_not_fond_error = "file not fond"
python_not_found_error = "python not found"
"""
shell exception
"""
je_editor_shell_error = "JEditor run on shell error"
"""
file exception
"""
je_editor_save_file_error = "JEditor save file error"
je_editor_open_file_error = "JEditor open file error"
je_editor_content_file_error = "JEditor content file error"
